/**************************************************************************************

        MUSTUXLIB - THE COMMON LIBRARY FOR ALL MUSTUX APPLICATIONS
        AUTHOR : See AUTHORS file for details

        This software is distributed under the terms of the GNU General Public License
        as specified in the COPYING file.

***************************************************************************************/

#ifndef MUSTUXJOGMOUSEBOARD_HH
#define MUSTUXJOGMOUSEBOARD_HH



#include <qevent.h>

#include "MustuxEngine.hh"

class MustuxInterface;

#include "MustuxJogMouseBoardAction.hh"

/** MustuxJogMouseBoard is one of the most special features of MustuxLib
 *  and the core of Mustux Project Philosophy. A Jog Mouse Board - for short JMB -
 *  is a engine that is able to recognize actions in the keyboard combined with
 *  mouse movements in order to avoid the traditional mouse-point-and-click paradigm
 *  of GUI applications. <br>
 *  In the traditional mouse-click paradigm , an user can perform 4 basic actions :
 *  click , double-click, click-and-drag, and right click.
 *  With JMB, there are at 16 kinds of actions multiplied by 101 keys of a standard keyboard.
 *  This means that once automated, an user can perform operations in MustuxApplications much
 *  faster than a traditional GUI environment. This is the base of Mustux Project paradigm and
 *  thats why some applications such Protux is beign called "the Emacs of the Audio".
 */
class MustuxJogMouseBoard : public MustuxEngine
        {
        Q_OBJECT
        public:
                /** create and build a JMB engine
                */
                MustuxJogMouseBoard(MustuxInterface* pInterface);
                ~MustuxJogMouseBoard();

                /** attach the JMB engine to an application (that will use and handle JMB events)
                *  Also, initialize the actions map, that is specified in a special file "jmb.map"
                *  (See MustuxJogMouseBoardEvent.hh for jmb.map details)
                */
                int init(QString mapFilename);

                /** activate the actions recognition engine and start responding to them
                */
                void activate();

                /** Suspends the engine . No action will be handled until activate() is called again
                */
                void suspend();

                /** finalize the engine and unbuild the actions map
                */
                int finalize();

                void reset();
                int init_map(QString mapFilename);

                void set_clear_time(int time);
                void set_hold_sensitiveness(int factor);
                void set_double_press_interval(int time);

                int catch_press(QKeyEvent* e);
                int catch_release(QKeyEvent* e);
		int get_collected_number();
		void stop_collecting();

                QString get_action_name(int a);
                int get_action_parameter(int a);
                
                
                void lock();
                void unlock();
                bool is_locked();


        protected :

                static const int STACK_SIZE = 4;
                static const int MAX_ACTIONS = 300;
                static const int JMB_KEY_PRESS = 1;
                static const int JMB_KEY_RELEASE = 2;

                static const int FKEY = 0;                 // <K>    - press one key fast
                static const int FKEY2 = 1;                // <KK>   - press two keys fast, together
                static const int HKEY = 2;                 // [K]    - Hold one key
                static const int HKEY2 = 3;                // [KK]   - Hold two keys, together
                static const int D_FKEY = 4;               // <<K>>  - double press one key fast
                static const int D_FKEY2 = 5;              // <<KK>> - double press two keys fast, together
                static const int FHKEY = 6;                // <[K]>  - press fast and then hold one key
                static const int FHKEY2 = 7;               // <[KK]> - press fast and then hold two keys together
                static const int S_FKEY_FKEY = 8;          // >K>K   - press one key and then other key fast and sequentially
                static const int S_FKEY_FKEY2 = 9;         // >K>KK  - press one key and then two other keys fast and sequentially
                static const int S_FKEY2_FKEY = 10;        // >KK>K  - press two keys and then another one fast and sequentially
                static const int S_FKEY2_FKEY2 = 11;       // >KK>KK  - press two keys and then another one fast and sequentially
                static const int S_FKEY_HKEY = 12;         // >K[K]  - press fastly one key and then hold another key, sequentially
                static const int S_FKEY_HKEY2 = 13;        // >K[KK] - press fastly one key and then hold two ther ones, sequentially
                static const int S_FKEY2_HKEY = 14;        // >KK[K] - press two keys together fastly and then hold a third one, sequentially
                static const int S_FKEY2_HKEY2 = 15;       // >KK[KK] - press two keys together fastly and then hold a third one, sequentially


                MustuxJogMouseBoardAction map[MAX_ACTIONS];
                MustuxInterface* assocInterface;
                int stackIndex;
                int eventType[STACK_SIZE];
                int eventStack[STACK_SIZE];
                int time[STACK_SIZE];
                int holdTime;
                int actionKey1A;
                int actionKey2A;
                int actionKey1B;
                int actionKey2B;
                int keyPressCounter;
                int pairOf2;
                int pairOf3;

                // JMB settings
                int clearTime;
                int holdSensitivenessFactor;
                int holdcheckInterval;
                int doublePressQuitTime;

                QTimer* holdTimer;
                QTimer* clearOutputTimer;
                QTimer* secondChanceTimer;
                int mapSize;
                bool active;
                bool isHolding;
                bool isKeyPressLocked;
                bool isTimerRunning;
                bool isHoldingOutput;
                bool isFirstPress;
                bool is_fake( int keyval);
                bool locked;

                int press1_k1;
                int press1_k2;
                int press2_k1;
                int press2_k2;
                int press1Type;
                int wholeAction;

                void push_event(int pType,  int pKey);
                void release_checker();
                int identify_first_press();
                int complement_first_press();
                void push_press( int k1 ,  int k2);
                void give_a_chance_for_second_press();
                void dispatch_action(int action);
                void dispatch_hold();
                void finish_hold();

                void conclusion();
                void hold_output();


        private:
                int uppercase(int k);
		QString sCollectedNumber;
		int collectedNumber;
		void check_number_collection();
		int collectedDigit;
		bool isCollecting;

        public slots:
                void holding();
                void quit_second_chance();
                void clear_output();

                // QEvent handlers
                void keyPressEvent ( QKeyEvent * );
                void keyReleaseEvent ( QKeyEvent * );



        };

#include "MustuxInterface.hh"



#endif


//eof


